{CrystalFontz 240x128 LCD display}

CON
  WMin  =       381                                                             'WAITCNT-expression-overhead Minimum

  
CON
        _clkmode = xtal1 + pll16x                                               'Standard clock mode * crystal frequency = 80 MHz
        _xinfreq = 5_000_000

  RS    =       8              'Register Select        H : instruction, L : data
  R_W   =       9              'Read Write             H : read       , L : write
  E     =       10              'Enable                 H : enabled    , L : disabled
  CS    =       11              'Chip Select            H : inactave   , L : active
  REST  =       12              'Reset                  H : normal opp , L : active

  DB0   =       0               'Data Bus Line 0
  DB7   =       7               'Data Bus Line 7

CON
  CharPerLine   = 40
  lines         = 16
  CurPos        = 7
  H_dots        = 240
  V_dots        = 128
  

VAR

PUB init
    reset  
    Write(%0000_0000, %0011_1000)                       {Display on, Master mode, Cursor blink}
    Write(%0000_0001, %0111_0101)                       {Character pitch 8x6}
    Write(%0000_0010, 38)                               {40-2 characters per line}
    Write(%0000_0011, 127)                              {display duty = 128}
    Write(%0000_0100, 7)                                {Cursor position}                       
    Write(%0000_1000, %0000_0000)                       {Display lower address}
    Write(%0000_1001, %0000_0000)                       {Display upper address}
    Write(%0000_1010, %0000_0000)                       {Cursor lower address}
    Write(%0000_1011, %0000_0000)                       {Cursor upper address}
    clr

PUB move(char, line) | CurAddress 'char = 1 to 40, line = 1 to 16
    CurAddress := ((line-1)*CharPerLine) + (char-1)
    Write(%0000_1010, CurAddress.byte[0])                       {Cursor lower address}
    Write(%0000_1011, CurAddress.byte[1])                       {Cursor upper address}
                 
PUB str(strAddr)
    repeat strsize(strAddr)                             ' for each character in string
      Write($0C,byte[strAddr++])                               '   write the character

PUB Dec(value) | i, x
{{Send value as decimal characters.
  Parameter:
    value - byte, word, or long value to send as decimal characters.}}

  x := value == NEGX                                                            'Check for max negative
  if value < 0
    value := ||(value+x)                                                        'If negative, make positive; adjust for max negative
    DisChar("-")                                                                   'and output sign

  i := 1_000_000_000                                                            'Initialize divisor

  repeat 10                                                                     'Loop for 10 digits
    if value => i                                                               
      DisChar(value / i + "0" + x*(i == 1))                                        'If non-zero digit, output digit; adjust for max negative
      value //= i                                                               'and digit from value
      result~~                                                                  'flag non-zero found
    elseif result or i == 1
      DisChar("0")                                                                 'If zero digit (or only digit) output it
    i /= 10                                                                     'Update divisor

PUB clr
    Write(%0000_1010, %0000_0000)                       {Cursor lower address}
    Write(%0000_1011, %0000_0000)                       {Cursor lower address}    
    repeat 640
      Write($0C,0)
    Write(%0000_1010, %0000_0000)                       {Cursor lower address}
    Write(%0000_1011, %0000_0000)                       {Cursor lower address}      

PUB DisChar (CharValue)
    Write ($0C, CharValue)
        
PUB Write (Instruction, Data)
    Command(Instruction)                        
     _OUTPUT(Data)
    Pulse(E)                       
    High(CS)

PUB Read (Instruction): Data
    Command(Instruction)
    HIGH(R_W)
    Pulse(E)
    dira[DB7..DB0]~
    Data := ina[DB7..DB0]
    High(CS)

PRI Command (Instruction)
    BUSY
    HIGH(RS)
    LOW(R_W)
    LOW(CS)
    _OUTPUT(Instruction)
    Pulse(E)
    LOW(RS)

PUB BUSY | IS_BUSY
    DIRA[DB7..DB0]~
    HIGH(RS)
    HIGH(R_W)
    LOW(CS)
      REPEAT
        HIGH(E)
        IS_BUSY := INA[DB7..DB0]
        LOW(E)
      WHILE (IS_BUSY & %1000_0000)
    HIGH(CS)                      
    DIRA[DB7..DB0]~~  

PUB PauseUSec(Duration) 
{{Pause execution in microseconds.
  PARAMETERS: Duration = number of microseconds to delay.
}}
    waitcnt(((clkfreq / 1_000_000 * Duration - 3928) #> WMin) + cnt)

PUB _OUTPUT(Data)
    dira[DB7..DB0]~~
    outa[DB7..DB0] := Data

PUB HIGH(Pin)
    dira[Pin]~~
    outa[Pin]~~
         
PUB LOW(Pin)
    dira[Pin]~~
    outa[Pin]~

PRI Pulse(Pin)
    HIGH(Pin)
    {PauseUSec(1)}              {was a pause, but display is faster than PROP}
    LOW(Pin)

PUB reset
    Low(REST)
    PauseUSec(1000)
    High(REST)        